
#[derive(Clone, Copy, Debug, PartialEq, Eq)]
pub enum RangeType {
	Open, Closed, Endless,
}

#[derive(Clone, Copy, Debug)]
pub struct Range {
	pub start: i64,
	pub stop: i64,
	pub ty: RangeType,
}

impl Range {
	pub fn len(&self) -> Option<usize> {
		match self.ty {
			RangeType::Open => Some((self.stop - self.start).max(0) as usize),
			RangeType::Closed => Some((self.stop - self.start + 1).max(0) as usize),
			RangeType::Endless => None,
		}
	}

	pub fn is_empty(&self) -> bool {
		match self.ty {
			RangeType::Open => self.stop - self.start <= 0,
			RangeType::Closed => self.stop - self.start < 0,
			RangeType::Endless => false,
		}
	}
}

impl IntoIterator for Range {
	type Item = i64;
	type IntoIter = Box<dyn Iterator<Item=i64>>;
	fn into_iter(self) -> Self::IntoIter {
		match self.ty {
			RangeType::Open => Box::new(self.start..self.stop),
			RangeType::Closed => Box::new(self.start..=self.stop),
			RangeType::Endless => Box::new(self.start..),
		}
	}
}


